@@ -19,6 +19,7 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 
 	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
+	controllerutils "github.com/openshift/hive/pkg/controller/utils"
 	testclaim "github.com/openshift/hive/pkg/test/clusterclaim"
 	testcd "github.com/openshift/hive/pkg/test/clusterdeployment"
 	testgeneric "github.com/openshift/hive/pkg/test/generic"
@@ -32,6 +33,10 @@ const (
 	passwordSecretName   = "password-secret"
 )
 
+func init() {
+	log.SetLevel(log.DebugLevel)
+}
+
 var (
 	subjects = []rbacv1.Subject{
 		{
@@ -80,18 +85,58 @@ func TestReconcileClusterClaim(t *testing.T) {
 		expectedRequeueAfter                   *time.Duration
 	}{
 		{
-			name:                 "new assignment",
-			claim:                claimBuilder.Build(testclaim.WithCluster(clusterName)),
-			cd:                   cdBuilder.Build(testcd.WithUnclaimedClusterPoolReference(claimNamespace, "test-pool")),
+			name:  "new assignment",
+			claim: claimBuilder.Build(testclaim.WithCluster(clusterName)),
+			cd: cdBuilder.Build(
+				testcd.WithUnclaimedClusterPoolReference(claimNamespace, "test-pool"),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.ClusterHibernatingCondition,
+					Status: corev1.ConditionTrue,
+				},
+				)),
 			expectCompletedClaim: true,
 			expectRBAC:           true,
+			expectedConditions: []hivev1.ClusterClaimCondition{
+				{
+					Type:    hivev1.ClusterClaimPendingCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "ClusterClaimed",
+					Message: "Cluster claimed",
+				},
+				{
+					Type:    hivev1.ClusterRunningCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "Resuming",
+					Message: "Waiting for cluster to be running",
+				},
+			},
 		},
 		{
-			name:                 "existing assignment",
-			claim:                claimBuilder.Build(testclaim.WithCluster(clusterName)),
-			cd:                   cdBuilder.Build(testcd.WithClusterPoolReference(claimNamespace, "test-pool", claimName)),
+			name:  "existing assignment",
+			claim: claimBuilder.Build(testclaim.WithCluster(clusterName)),
+			cd: cdBuilder.Build(
+				testcd.WithClusterPoolReference(claimNamespace, "test-pool", claimName),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.ClusterHibernatingCondition,
+					Status: corev1.ConditionTrue,
+				}),
+			),
 			expectCompletedClaim: true,
 			expectRBAC:           true,
+			expectedConditions: []hivev1.ClusterClaimCondition{
+				{
+					Type:    hivev1.ClusterClaimPendingCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "ClusterClaimed",
+					Message: "Cluster claimed",
+				},
+				{
+					Type:    hivev1.ClusterRunningCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "Resuming",
+					Message: "Waiting for cluster to be running",
+				},
+			},
 		},
 		{
 			name:               "assignment conflict",
@@ -127,14 +172,28 @@ func TestReconcileClusterClaim(t *testing.T) {
 					},
 				),
 			),
-			cd:                   cdBuilder.Build(testcd.WithUnclaimedClusterPoolReference(claimNamespace, "test-pool")),
+			cd: cdBuilder.Build(
+				testcd.WithUnclaimedClusterPoolReference(claimNamespace, "test-pool"),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.ClusterHibernatingCondition,
+					Status: corev1.ConditionTrue,
+				}),
+			),
 			expectCompletedClaim: true,
-			expectedConditions: []hivev1.ClusterClaimCondition{{
-				Type:    hivev1.ClusterClaimPendingCondition,
-				Status:  corev1.ConditionFalse,
-				Reason:  "ClusterClaimed",
-				Message: "Cluster claimed",
-			}},
+			expectedConditions: []hivev1.ClusterClaimCondition{
+				{
+					Type:    hivev1.ClusterClaimPendingCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "ClusterClaimed",
+					Message: "Cluster claimed",
+				},
+				{
+					Type:    hivev1.ClusterRunningCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "Resuming",
+					Message: "Waiting for cluster to be running",
+				},
+			},
 			expectRBAC: true,
 		},
 		{
@@ -210,15 +269,40 @@ func TestReconcileClusterClaim(t *testing.T) {
 			expectNoFinalizer: true,
 		},
 		{
-			name:                 "no RBAC when no subjects",
-			claim:                claimBuilder.Build(testclaim.WithCluster(clusterName), testclaim.WithSubjects(nil)),
-			cd:                   cdBuilder.Build(testcd.WithUnclaimedClusterPoolReference(claimNamespace, "test-pool")),
+			name:  "no RBAC when no subjects",
+			claim: claimBuilder.Build(testclaim.WithCluster(clusterName), testclaim.WithSubjects(nil)),
+			cd: cdBuilder.Build(testcd.WithUnclaimedClusterPoolReference(claimNamespace, "test-pool"),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.ClusterHibernatingCondition,
+					Status: corev1.ConditionTrue,
+				}),
+			),
 			expectCompletedClaim: true,
+			expectedConditions: []hivev1.ClusterClaimCondition{
+				{
+					Type:    hivev1.ClusterClaimPendingCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "ClusterClaimed",
+					Message: "Cluster claimed",
+				},
+				{
+					Type:    hivev1.ClusterRunningCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "Resuming",
+					Message: "Waiting for cluster to be running",
+				},
+			},
 		},
 		{
 			name:  "update existing role",
 			claim: claimBuilder.Build(testclaim.WithCluster(clusterName)),
-			cd:    cdBuilder.Build(testcd.WithUnclaimedClusterPoolReference(claimNamespace, "test-pool")),
+			cd: cdBuilder.Build(
+				testcd.WithUnclaimedClusterPoolReference(claimNamespace, "test-pool"),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.ClusterHibernatingCondition,
+					Status: corev1.ConditionTrue,
+				}),
+			),
 			existing: []runtime.Object{
 				func() runtime.Object {
 					r := testRole()
@@ -228,11 +312,31 @@ func TestReconcileClusterClaim(t *testing.T) {
 			},
 			expectCompletedClaim: true,
 			expectRBAC:           true,
+			expectedConditions: []hivev1.ClusterClaimCondition{
+				{
+					Type:    hivev1.ClusterClaimPendingCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "ClusterClaimed",
+					Message: "Cluster claimed",
+				},
+				{
+					Type:    hivev1.ClusterRunningCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "Resuming",
+					Message: "Waiting for cluster to be running",
+				},
+			},
 		},
 		{
 			name:  "update existing rolebinding",
 			claim: claimBuilder.Build(testclaim.WithCluster(clusterName)),
-			cd:    cdBuilder.Build(testcd.WithUnclaimedClusterPoolReference(claimNamespace, "test-pool")),
+			cd: cdBuilder.Build(
+				testcd.WithUnclaimedClusterPoolReference(claimNamespace, "test-pool"),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.ClusterHibernatingCondition,
+					Status: corev1.ConditionTrue,
+				}),
+			),
 			existing: []runtime.Object{
 				func() runtime.Object {
 					rb := testRoleBinding()
@@ -243,6 +347,20 @@ func TestReconcileClusterClaim(t *testing.T) {
 			},
 			expectCompletedClaim: true,
 			expectRBAC:           true,
+			expectedConditions: []hivev1.ClusterClaimCondition{
+				{
+					Type:    hivev1.ClusterClaimPendingCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "ClusterClaimed",
+					Message: "Cluster claimed",
+				},
+				{
+					Type:    hivev1.ClusterRunningCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "Resuming",
+					Message: "Waiting for cluster to be running",
+				},
+			},
 		},
 		{
 			name:  "new assignment bring cluster out of hibernation",
@@ -250,10 +368,28 @@ func TestReconcileClusterClaim(t *testing.T) {
 			cd: cdBuilder.Build(
 				testcd.WithUnclaimedClusterPoolReference(claimNamespace, "test-pool"),
 				testcd.WithPowerState(hivev1.HibernatingClusterPowerState),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.ClusterHibernatingCondition,
+					Status: corev1.ConditionTrue,
+				}),
 			),
 			expectCompletedClaim: true,
 			expectRBAC:           true,
 			expectHibernating:    false,
+			expectedConditions: []hivev1.ClusterClaimCondition{
+				{
+					Type:    hivev1.ClusterClaimPendingCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "ClusterClaimed",
+					Message: "Cluster claimed",
+				},
+				{
+					Type:    hivev1.ClusterRunningCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "Resuming",
+					Message: "Waiting for cluster to be running",
+				},
+			},
 		},
 		{
 			name:  "existing assignment does not change power state",
@@ -261,10 +397,28 @@ func TestReconcileClusterClaim(t *testing.T) {
 			cd: cdBuilder.Build(
 				testcd.WithClusterPoolReference(claimNamespace, "test-pool", claimName),
 				testcd.WithPowerState(hivev1.HibernatingClusterPowerState),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.ClusterHibernatingCondition,
+					Status: corev1.ConditionTrue,
+				}),
 			),
 			expectCompletedClaim: true,
 			expectRBAC:           true,
 			expectHibernating:    true,
+			expectedConditions: []hivev1.ClusterClaimCondition{
+				{
+					Type:    hivev1.ClusterClaimPendingCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "ClusterClaimed",
+					Message: "Cluster claimed",
+				},
+				{
+					Type:    hivev1.ClusterRunningCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "Resuming",
+					Message: "Waiting for cluster to be running",
+				},
+			},
 		},
 		{
 			name: "claim with elapsed lifetime is deleted",
@@ -277,8 +431,28 @@ func TestReconcileClusterClaim(t *testing.T) {
 					LastTransitionTime: metav1.NewTime(time.Now().Add(-1 * time.Hour)),
 				}),
 			),
-			cd:            cdBuilder.Build(testcd.WithClusterPoolReference(claimNamespace, "test-pool", claimName)),
+			cd: cdBuilder.Build(
+				testcd.WithClusterPoolReference(claimNamespace, "test-pool", claimName),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.ClusterHibernatingCondition,
+					Status: corev1.ConditionTrue,
+				}),
+			),
 			expectDeleted: true,
+			expectedConditions: []hivev1.ClusterClaimCondition{
+				{
+					Type:    hivev1.ClusterClaimPendingCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "ClusterClaimed",
+					Message: "Cluster claimed",
+				},
+				{
+					Type:    hivev1.ClusterRunningCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "Resuming",
+					Message: "Waiting for cluster to be running",
+				},
+			},
 		},
 		{
 			name: "claim with non-elapsed lifetime is not deleted",
@@ -293,18 +467,32 @@ func TestReconcileClusterClaim(t *testing.T) {
 					LastTransitionTime: metav1.NewTime(time.Now().Add(-1 * time.Hour)),
 				}),
 			),
-			cd: cdBuilder.Build(testcd.WithClusterPoolReference(claimNamespace, "test-pool", claimName)),
+			cd: cdBuilder.Build(
+				testcd.WithClusterPoolReference(claimNamespace, "test-pool", claimName),
+				testcd.WithCondition(hivev1.ClusterDeploymentCondition{
+					Type:   hivev1.ClusterHibernatingCondition,
+					Status: corev1.ConditionTrue,
+				}),
+			),
 			existing: []runtime.Object{
 				testRole(),
 				testRoleBinding(),
 			},
 			expectCompletedClaim: true,
-			expectedConditions: []hivev1.ClusterClaimCondition{{
-				Type:    hivev1.ClusterClaimPendingCondition,
-				Status:  corev1.ConditionFalse,
-				Reason:  "ClusterClaimed",
-				Message: "Cluster claimed",
-			}},
+			expectedConditions: []hivev1.ClusterClaimCondition{
+				{
+					Type:    hivev1.ClusterClaimPendingCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "ClusterClaimed",
+					Message: "Cluster claimed",
+				},
+				{
+					Type:    hivev1.ClusterRunningCondition,
+					Status:  corev1.ConditionFalse,
+					Reason:  "Resuming",
+					Message: "Waiting for cluster to be running",
+				},
+			},
 			expectRBAC:           true,
 			expectedRequeueAfter: func(d time.Duration) *time.Duration { return &d }(2 * time.Hour),
 		},
@@ -365,7 +553,8 @@ func TestReconcileClusterClaim(t *testing.T) {
 					assert.NotEqual(t, claimName, cd.Spec.ClusterPoolRef.ClaimName, "expected ClusterDeployment to not be claimed by ClusterClaim")
 				}
 				if isAssignedCD {
-					assignedClusterDeploymentExists = true
+					toRemove := controllerutils.IsClaimedClusterMarkedForRemoval(&cd)
+					assignedClusterDeploymentExists = !toRemove
 					if test.expectHibernating {
 						assert.Equal(t, hivev1.HibernatingClusterPowerState, cd.Spec.PowerState, "expected ClusterDeployment to be hibernating")
 					} else {
