@@ -3,15 +3,16 @@ package clusterpool
 import (
 	"context"
 	"fmt"
+	"math"
 	"reflect"
 	"sort"
-	"time"
 
 	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
 	utilerrors "k8s.io/apimachinery/pkg/util/errors"
 
 	corev1 "k8s.io/api/core/v1"
+	rbacv1 "k8s.io/api/rbac/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
@@ -34,11 +35,14 @@ import (
 )
 
 const (
-	ControllerName             = hivev1.ClusterpoolControllerName
-	finalizer                  = "hive.openshift.io/clusters"
-	imageSetDependent          = "cluster image set"
-	pullSecretDependent        = "pull secret"
-	credentialsSecretDependent = "credentials secret"
+	ControllerName                  = hivev1.ClusterpoolControllerName
+	finalizer                       = "hive.openshift.io/clusters"
+	imageSetDependent               = "cluster image set"
+	pullSecretDependent             = "pull secret"
+	credentialsSecretDependent      = "credentials secret"
+	clusterPoolAdminRoleName        = "hive-cluster-pool-admin"
+	clusterPoolAdminRoleBindingName = "hive-cluster-pool-admin-binding"
+	icSecretDependent               = "install config template secret"
 )
 
 var (
@@ -112,9 +116,76 @@ func AddToManager(mgr manager.Manager, r *ReconcileClusterPool, concurrentReconc
 		return err
 	}
 
+	// Watch for changes to the hive cluster pool admin RoleBindings
+	if err := c.Watch(
+		&source.Kind{Type: &rbacv1.RoleBinding{}},
+		&handler.EnqueueRequestsFromMapFunc{
+			ToRequests: requestsForRBACResources(r.Client, r.logger),
+		},
+	); err != nil {
+		return err
+	}
+
+	// Watch for changes to the hive-cluster-pool-admin-binding RoleBinding
+	if err := c.Watch(
+		&source.Kind{Type: &rbacv1.RoleBinding{}},
+		&handler.EnqueueRequestsFromMapFunc{
+			ToRequests: requestsForCDRBACResources(r.Client, clusterPoolAdminRoleBindingName, r.logger),
+		},
+	); err != nil {
+		return err
+	}
+
 	return nil
 }
 
+func requestsForCDRBACResources(c client.Client, resourceName string, logger log.FieldLogger) handler.ToRequestsFunc {
+	return func(o handler.MapObject) []reconcile.Request {
+		if o.Meta.GetName() != resourceName {
+			return nil
+		}
+		clusterName := o.Meta.GetNamespace()
+		cd := &hivev1.ClusterDeployment{}
+		if err := c.Get(context.Background(), client.ObjectKey{Namespace: clusterName, Name: clusterName}, cd); err != nil {
+			logger.WithError(err).Log(controllerutils.LogLevel(err), "failed to get ClusterDeployment for RBAC resource")
+			return nil
+		}
+		cpKey := clusterPoolKey(cd)
+		if cpKey == nil {
+			return nil
+		}
+		return []reconcile.Request{{NamespacedName: *cpKey}}
+	}
+}
+
+func requestsForRBACResources(c client.Client, logger log.FieldLogger) handler.ToRequestsFunc {
+	return func(o handler.MapObject) []reconcile.Request {
+		binding, ok := o.Object.(*rbacv1.RoleBinding)
+		if !ok {
+			return nil
+		}
+		if binding.RoleRef.Kind != "ClusterRole" || binding.RoleRef.Name != clusterPoolAdminRoleName {
+			return nil
+		}
+
+		cpList := &hivev1.ClusterPoolList{}
+		if err := c.List(context.Background(), cpList, client.InNamespace(o.Meta.GetNamespace())); err != nil {
+			logger.WithError(err).Log(controllerutils.LogLevel(err), "failed to list cluster pools for RBAC resource")
+			return nil
+		}
+		var requests []reconcile.Request
+		for _, cpl := range cpList.Items {
+			requests = append(requests, reconcile.Request{
+				NamespacedName: types.NamespacedName{
+					Namespace: cpl.Namespace,
+					Name:      cpl.Name,
+				},
+			})
+		}
+		return requests
+	}
+}
+
 var _ reconcile.Reconciler = &ReconcileClusterPool{}
 
 // ReconcileClusterPool reconciles a ClusterPool object
@@ -128,15 +199,10 @@ type ReconcileClusterPool struct {
 // Reconcile reads the state of the ClusterPool, checks if we currently have enough ClusterDeployments waiting, and
 // attempts to reach the desired state if not.
 func (r *ReconcileClusterPool) Reconcile(request reconcile.Request) (reconcile.Result, error) {
-	start := time.Now()
-	logger := r.logger.WithField("clusterPool", request.Name)
-
+	logger := controllerutils.BuildControllerLogger(ControllerName, "clusterPool", request.NamespacedName)
 	logger.Infof("reconciling cluster pool")
-	defer func() {
-		dur := time.Since(start)
-		hivemetrics.MetricControllerReconcileTime.WithLabelValues(ControllerName.String()).Observe(dur.Seconds())
-		logger.WithField("elapsed", dur).Info("reconcile complete")
-	}()
+	recobsrv := hivemetrics.NewReconcileObserver(ControllerName, logger)
+	defer recobsrv.ObserveControllerReconcileTime()
 
 	// Fetch the ClusterPool instance
 	clp := &hivev1.ClusterPool{}
@@ -173,15 +239,27 @@ func (r *ReconcileClusterPool) Reconcile(request reconcile.Request) (reconcile.R
 	}
 
 	// Find all ClusterDeployments from this pool:
-	poolCDs, err := r.getAllUnclaimedClusterDeployments(clp, logger)
+	claimedCDs, unClaminedCDs, err := r.getAllClusterDeploymentsForPool(clp, logger)
 	if err != nil {
 		return reconcile.Result{}, err
 	}
 
+	var toRemoveClaimedCDs []*hivev1.ClusterDeployment
+	numberOfDeletingClaimedCDs := 0
+	for _, cd := range claimedCDs {
+		toRemove := controllerutils.IsClaimedClusterMarkedForRemoval(cd)
+		switch {
+		case cd.DeletionTimestamp != nil:
+			numberOfDeletingClaimedCDs++
+		case toRemove:
+			toRemoveClaimedCDs = append(toRemoveClaimedCDs, cd)
+		}
+	}
+
 	var installingCDs []*hivev1.ClusterDeployment
 	var readyCDs []*hivev1.ClusterDeployment
 	numberOfDeletingCDs := 0
-	for _, cd := range poolCDs {
+	for _, cd := range unClaminedCDs {
 		switch {
 		case cd.DeletionTimestamp != nil:
 			numberOfDeletingCDs++
@@ -195,7 +273,7 @@ func (r *ReconcileClusterPool) Reconcile(request reconcile.Request) (reconcile.R
 	logger.WithFields(log.Fields{
 		"installing": len(installingCDs),
 		"deleting":   numberOfDeletingCDs,
-		"total":      len(poolCDs),
+		"total":      len(unClaminedCDs),
 		"ready":      len(readyCDs),
 	}).Debug("found clusters for ClusterPool")
 
@@ -209,6 +287,22 @@ func (r *ReconcileClusterPool) Reconcile(request reconcile.Request) (reconcile.R
 		}
 	}
 
+	availableCapacity := math.MaxInt32
+	if clp.Spec.MaxSize != nil {
+		availableCapacity = int(*clp.Spec.MaxSize) - len(unClaminedCDs) - len(claimedCDs)
+		if availableCapacity <= 0 {
+			logger.WithFields(log.Fields{
+				"lUnclaimedSize": len(unClaminedCDs),
+				"ClaimedSize":    len(claimedCDs),
+				"Capacity":       *clp.Spec.MaxSize,
+			}).Info("Cannot add more clusters because no capacity available.")
+		}
+	}
+	if err := r.setAvailableCapacityCondition(clp, availableCapacity > 0, logger); err != nil {
+		logger.WithError(err).Error("error setting CapacityAvailable condition")
+		return reconcile.Result{}, err
+	}
+
 	pendingClaims, err := r.getAllPendingClusterClaims(clp, logger)
 	if err != nil {
 		return reconcile.Result{}, err
@@ -223,23 +317,167 @@ func (r *ReconcileClusterPool) Reconcile(request reconcile.Request) (reconcile.R
 		return reconcile.Result{}, err
 	}
 
+	availableCurrent := math.MaxInt32
+	if clp.Spec.MaxConcurrent != nil {
+		availableCurrent = int(*clp.Spec.MaxConcurrent) - len(installingCDs) - numberOfDeletingCDs - numberOfDeletingClaimedCDs
+		if availableCurrent < 0 {
+			availableCurrent = 0
+		}
+	}
+
+	// remove clusters that were previously claimed but now not required.
+	toDel := minIntVarible(len(toRemoveClaimedCDs), availableCurrent)
+	for _, cd := range toRemoveClaimedCDs[:toDel] {
+		cdLog := logger.WithField("cluster", cd.Name)
+		cdLog.Info("deleting cluster deployment for previous claim")
+		if err := r.Client.Delete(context.Background(), cd); err != nil {
+			cdLog.WithError(err).Error("error deleting cluster deployment")
+			return reconcile.Result{}, err
+		}
+	}
+	availableCurrent -= toDel
+
 	switch drift := reserveSize - int(clp.Spec.Size); {
+	// activity quota exceeded, so no action
+	case availableCurrent <= 0:
+		logger.WithFields(log.Fields{
+			"MaxConcurrent": *clp.Spec.MaxConcurrent,
+			"Available":     availableCurrent,
+		}).Info("Cannot create/delete clusters as max concurrent quota exceeded.")
 	// If too many, delete some.
 	case drift > 0:
-		if err := r.deleteExcessClusters(installingCDs, readyCDs, drift, logger); err != nil {
+		toDel := minIntVarible(drift, availableCurrent)
+		if err := r.deleteExcessClusters(installingCDs, readyCDs, toDel, logger); err != nil {
 			return reconcile.Result{}, err
 		}
 	// If too few, create new InstallConfig and ClusterDeployment.
 	case drift < 0:
-		if err := r.addClusters(clp, -drift, logger); err != nil {
+		if availableCapacity <= 0 {
+			break
+		}
+		toAdd := minIntVarible(-drift, availableCapacity, availableCurrent)
+		if err := r.addClusters(clp, toAdd, logger); err != nil {
 			log.WithError(err).Error("error adding clusters")
 			return reconcile.Result{}, err
 		}
 	}
 
+	if err := r.reconcileRBAC(clp, logger); err != nil {
+		log.WithError(err).Error("error reconciling RBAC")
+		return reconcile.Result{}, err
+	}
+
 	return reconcile.Result{}, nil
 }
 
+func minIntVarible(v1 int, vn ...int) (m int) {
+	m = v1
+	for i := 0; i < len(vn); i++ {
+		if vn[i] < m {
+			m = vn[i]
+		}
+	}
+	return
+}
+
+func (r *ReconcileClusterPool) reconcileRBAC(
+	clp *hivev1.ClusterPool,
+	logger log.FieldLogger,
+) error {
+	rbList := &rbacv1.RoleBindingList{}
+	if err := r.Client.List(context.Background(), rbList, client.InNamespace(clp.GetNamespace())); err != nil {
+		log.WithError(err).Error("error listing rolebindings")
+		return err
+	}
+	var subs []rbacv1.Subject
+	var roleRef rbacv1.RoleRef
+	for _, rb := range rbList.Items {
+		if rb.RoleRef.Kind != "ClusterRole" ||
+			rb.RoleRef.Name != clusterPoolAdminRoleName {
+			continue
+		}
+		roleRef = rb.RoleRef
+		subs = append(subs, rb.Subjects...)
+	}
+
+	if len(subs) == 0 {
+		// nothing to do here.
+		return nil
+	}
+
+	// get namespaces where role needs to be bound.
+	cdNSList := &corev1.NamespaceList{}
+	if err := r.Client.List(context.Background(), cdNSList,
+		client.MatchingLabels{constants.ClusterPoolNameLabel: clp.GetName()}); err != nil {
+		log.WithError(err).Error("error listing namespaces for cluster pool")
+		return err
+	}
+
+	// create role bindings.
+	var errs []error
+	for _, ns := range cdNSList.Items {
+		if ns.DeletionTimestamp != nil {
+			logger.WithField("namespace", ns.GetName()).
+				Debug("skipping syncing the rolebindings as the namespace is marked for deletion")
+			continue
+		}
+		rb := &rbacv1.RoleBinding{
+			ObjectMeta: metav1.ObjectMeta{
+				Namespace: ns.GetName(),
+				Name:      clusterPoolAdminRoleBindingName,
+			},
+			Subjects: subs,
+			RoleRef:  roleRef,
+		}
+		orb := &rbacv1.RoleBinding{}
+		if err := r.applyRoleBinding(rb, orb, logger); err != nil {
+			errs = append(errs, errors.Wrapf(err, "failed to apply rolebinding in namespace %s", ns.GetNamespace()))
+			continue
+		}
+	}
+	if err := utilerrors.NewAggregate(errs); err != nil {
+		return err
+	}
+	return nil
+}
+
+func (r *ReconcileClusterPool) applyRoleBinding(desired, observed *rbacv1.RoleBinding, logger log.FieldLogger) error {
+	key := client.ObjectKey{Namespace: desired.GetNamespace(), Name: desired.GetName()}
+	logger = logger.WithField("rolebinding", key)
+
+	switch err := r.Client.Get(context.Background(), key, observed); {
+	case apierrors.IsNotFound(err):
+		logger.Info("creating rolebinding")
+		if err := r.Create(context.Background(), desired); err != nil {
+			logger.WithError(err).Error("could not create rolebinding")
+			return err
+		}
+		return nil
+	case err != nil:
+		logger.WithError(err).Error("error getting rolebinding")
+		return errors.Wrap(err, "could not get rolebinding")
+	}
+
+	if reflect.DeepEqual(observed.Subjects, desired.Subjects) && reflect.DeepEqual(observed.RoleRef, desired.RoleRef) {
+		return nil
+	}
+	obsrvdSubs := observed.Subjects
+	observed.Subjects = desired.Subjects
+	observed.RoleRef = desired.RoleRef
+
+	logger.WithFields(log.Fields{
+		"observedSubjects": obsrvdSubs,
+		"desiredSubjects":  desired.Subjects,
+		"desiredRoleRef":   desired.RoleRef,
+	}).Info("updating rolebinding")
+	if err := r.Update(context.Background(), observed); err != nil {
+		logger.WithError(err).Error("could not update rolebinding")
+		return err
+	}
+
+	return nil
+}
+
 func (r *ReconcileClusterPool) addClusters(
 	clp *hivev1.ClusterPool,
 	newClusterCount int,
@@ -259,6 +497,12 @@ func (r *ReconcileClusterPool) addClusters(
 		errs = append(errs, fmt.Errorf("%s: %w", pullSecretDependent, err))
 	}
 
+	// Load install config templates if specified
+	installConfigTemplate, err := r.getInstallConfigTemplate(clp, logger)
+	if err != nil {
+		errs = append(errs, fmt.Errorf("%s: %w", icSecretDependent, err))
+	}
+
 	cloudBuilder, err := r.createCloudBuilder(clp, logger)
 	if err != nil {
 		errs = append(errs, fmt.Errorf("%s: %w", credentialsSecretDependent, err))
@@ -275,7 +519,7 @@ func (r *ReconcileClusterPool) addClusters(
 	}
 
 	for i := 0; i < newClusterCount; i++ {
-		if err := r.createCluster(clp, cloudBuilder, pullSecret, logger); err != nil {
+		if err := r.createCluster(clp, cloudBuilder, pullSecret, installConfigTemplate, logger); err != nil {
 			return err
 		}
 	}
@@ -287,6 +531,7 @@ func (r *ReconcileClusterPool) createCluster(
 	clp *hivev1.ClusterPool,
 	cloudBuilder clusterresource.CloudBuilder,
 	pullSecret string,
+	installConfigTemplate string,
 	logger log.FieldLogger,
 ) error {
 	ns, err := r.createRandomNamespace(clp)
@@ -298,21 +543,28 @@ func (r *ReconcileClusterPool) createCluster(
 
 	// We will use this unique random namespace name for our cluster name.
 	builder := &clusterresource.Builder{
-		Name:             ns.Name,
-		Namespace:        ns.Name,
-		BaseDomain:       clp.Spec.BaseDomain,
-		ImageSet:         clp.Spec.ImageSetRef.Name,
-		WorkerNodesCount: int64(3),
-		MachineNetwork:   "10.0.0.0/16",
-		PullSecret:       pullSecret,
-		CloudBuilder:     cloudBuilder,
-		Labels:           clp.Spec.Labels,
+		Name:                  ns.Name,
+		Namespace:             ns.Name,
+		BaseDomain:            clp.Spec.BaseDomain,
+		ImageSet:              clp.Spec.ImageSetRef.Name,
+		WorkerNodesCount:      int64(3),
+		MachineNetwork:        "10.0.0.0/16",
+		PullSecret:            pullSecret,
+		CloudBuilder:          cloudBuilder,
+		Labels:                clp.Spec.Labels,
+		InstallConfigTemplate: installConfigTemplate,
+		SkipMachinePools:      clp.Spec.SkipMachinePools,
+	}
+
+	if clp.Spec.HibernateAfter != nil {
+		builder.HibernateAfter = &clp.Spec.HibernateAfter.Duration
 	}
 
 	objs, err := builder.Build()
 	if err != nil {
 		return errors.Wrap(err, "error building resources")
 	}
+
 	poolKey := types.NamespacedName{Namespace: clp.Namespace, Name: clp.Name}.String()
 	r.expectations.ExpectCreations(poolKey, 1)
 	// Add the ClusterPoolRef to the ClusterDeployment, and move it to the end of the slice.
@@ -398,11 +650,11 @@ func (r *ReconcileClusterPool) reconcileDeletedPool(pool *hivev1.ClusterPool, lo
 	if !controllerutils.HasFinalizer(pool, finalizer) {
 		return nil
 	}
-	poolCDs, err := r.getAllUnclaimedClusterDeployments(pool, logger)
+	_, unClaimedCDs, err := r.getAllClusterDeploymentsForPool(pool, logger)
 	if err != nil {
 		return err
 	}
-	for _, cd := range poolCDs {
+	for _, cd := range unClaimedCDs {
 		if cd.DeletionTimestamp != nil {
 			continue
 		}
@@ -419,19 +671,24 @@ func (r *ReconcileClusterPool) reconcileDeletedPool(pool *hivev1.ClusterPool, lo
 	return nil
 }
 
-func (r *ReconcileClusterPool) getAllUnclaimedClusterDeployments(pool *hivev1.ClusterPool, logger log.FieldLogger) ([]*hivev1.ClusterDeployment, error) {
+func (r *ReconcileClusterPool) getAllClusterDeploymentsForPool(pool *hivev1.ClusterPool, logger log.FieldLogger) (claimed, unclaimed []*hivev1.ClusterDeployment, err error) {
 	cdList := &hivev1.ClusterDeploymentList{}
 	if err := r.Client.List(context.Background(), cdList); err != nil {
 		logger.WithError(err).Error("error listing ClusterDeployments")
-		return nil, err
+		return nil, nil, err
 	}
-	var poolCDs []*hivev1.ClusterDeployment
 	for i, cd := range cdList.Items {
-		if refInCD := cd.Spec.ClusterPoolRef; refInCD != nil && *refInCD == poolReference(pool) {
-			poolCDs = append(poolCDs, &cdList.Items[i])
+		if refInCD := cd.Spec.ClusterPoolRef; refInCD != nil {
+			if refInCD.Namespace == pool.Namespace && refInCD.PoolName == pool.Name {
+				if len(refInCD.ClaimName) > 0 {
+					claimed = append(claimed, &cdList.Items[i])
+				} else {
+					unclaimed = append(unclaimed, &cdList.Items[i])
+				}
+			}
 		}
 	}
-	return poolCDs, nil
+	return claimed, unclaimed, nil
 }
 
 func poolReference(pool *hivev1.ClusterPool) hivev1.ClusterPoolReference {
@@ -483,6 +740,35 @@ func (r *ReconcileClusterPool) setMissingDependenciesCondition(pool *hivev1.Clus
 	return nil
 }
 
+func (r *ReconcileClusterPool) setAvailableCapacityCondition(pool *hivev1.ClusterPool, available bool, logger log.FieldLogger) error {
+	status := corev1.ConditionTrue
+	reason := "Available"
+	message := "There is capacity to add more clusters to the pool."
+	updateConditionCheck := controllerutils.UpdateConditionNever
+	if !available {
+		status = corev1.ConditionFalse
+		reason = "MaxCapacity"
+		message = fmt.Sprintf("Pool is at maximum capacity of %d waiting and claimed clusters.", *pool.Spec.MaxSize)
+		updateConditionCheck = controllerutils.UpdateConditionIfReasonOrMessageChange
+	}
+	conds, changed := controllerutils.SetClusterPoolConditionWithChangeCheck(
+		pool.Status.Conditions,
+		hivev1.ClusterPoolCapacityAvailableCondition,
+		status,
+		reason,
+		message,
+		updateConditionCheck,
+	)
+	if changed {
+		pool.Status.Conditions = conds
+		if err := r.Status().Update(context.Background(), pool); err != nil {
+			logger.WithError(err).Log(controllerutils.LogLevel(err), "could not update ClusterPool conditions")
+			return errors.Wrap(err, "could not update ClusterPool conditions")
+		}
+	}
+	return nil
+}
+
 func (r *ReconcileClusterPool) verifyClusterImageSet(pool *hivev1.ClusterPool, logger log.FieldLogger) error {
 	err := r.Get(context.Background(), client.ObjectKey{Name: pool.Spec.ImageSetRef.Name}, &hivev1.ClusterImageSet{})
 	if err != nil {
@@ -491,6 +777,29 @@ func (r *ReconcileClusterPool) verifyClusterImageSet(pool *hivev1.ClusterPool, l
 	return err
 }
 
+func (r *ReconcileClusterPool) getInstallConfigTemplate(pool *hivev1.ClusterPool, logger log.FieldLogger) (string, error) {
+	if pool.Spec.InstallConfigSecretTemplateRef == nil {
+		return "", nil
+	}
+	installConfigSecret := &corev1.Secret{}
+	err := r.Client.Get(
+		context.Background(),
+		types.NamespacedName{Namespace: pool.Namespace, Name: pool.Spec.InstallConfigSecretTemplateRef.Name},
+		installConfigSecret,
+	)
+	if err != nil {
+		logger.WithError(err).Log(controllerutils.LogLevel(err), "error reading install-config secret")
+		return "", err
+	}
+	installConfig, ok := installConfigSecret.Data["install-config.yaml"]
+	if !ok {
+		logger.Info("install-config secret does not contain install-config.yaml data")
+		return "", errors.New("install-config secret does not contain install-config.yaml data")
+	}
+	return string(installConfig), nil
+
+}
+
 func (r *ReconcileClusterPool) getPullSecret(pool *hivev1.ClusterPool, logger log.FieldLogger) (string, error) {
 	if pool.Spec.PullSecretRef == nil {
 		return "", nil
