@@ -169,16 +169,10 @@ type ReconcileBackup struct {
 
 // Reconcile ensures that all Hive object changes have corresponding Velero backup objects.
 func (r *ReconcileBackup) Reconcile(request reconcile.Request) (reconcile.Result, error) {
-	start := time.Now()
-	nsLogger := r.logger.WithField("namespace", request.Namespace)
-
-	// For logging, we need to see when the reconciliation loop starts and ends.
+	nsLogger := controllerutils.BuildControllerLogger(ControllerName, "namespace", request.NamespacedName)
 	nsLogger.Info("reconciling backups and Hive object changes")
-	defer func() {
-		dur := time.Since(start)
-		hivemetrics.MetricControllerReconcileTime.WithLabelValues(ControllerName.String()).Observe(dur.Seconds())
-		nsLogger.WithField("elapsed", dur).Info("reconcile complete")
-	}()
+	recobsrv := hivemetrics.NewReconcileObserver(ControllerName, nsLogger)
+	defer recobsrv.ObserveControllerReconcileTime()
 
 	cp, checkpointFound, err := r.getNamespaceCheckpoint(request.Namespace, nsLogger)
 	if err != nil {
